[VERTEX]
#version 430
layout(location=0) in vec2 position;
layout(location = 3) in vec2 texcoord;
layout(location = 5) in vec2 vertex_color;

out vec2 fragment_uv;

out vec3 dir;
out vec3 pos;

uniform vec3 dir3;
uniform vec3 pos3;
uniform vec4 quat_in;
uniform float fov;
uniform float aspect;

vec3 qtransform(vec4 q, vec3 v){
	return v + 2.0*cross(cross(v, q.xyz) + q.w*v, q.xyz);
}

void main()
{

	fragment_uv = texcoord * 2.0 - vec2(1.0);
	fragment_uv *= vec2(aspect, 1.0);

	dir = qtransform(quat_in, vec3(fragment_uv * fov, 1.0));
	pos = pos3;
	gl_Position = vec4(position, 0.0, 1.0);
}

[FRAGMENT]
#version 430
uniform samplerCube texture0;
#define MAX_LIGHTS 4;
#define MAX_SPHERES 1;
struct Light
{
	vec4 position;
	vec4 color;
};

struct Sphere
{
	vec4 position;
	vec4 color;
};

layout(binding = 0, std140) uniform Lights{
	Light lights[4];
};

layout(binding = 1, std140) uniform Spheres{
	Sphere spheres[9];
};

float intersect_ray_sphere(vec3 from, vec3 dir, vec4 sphere, out vec3 new_pos, out vec3 new_normal)
{
	vec3 v = from - sphere.xyz;
	float B = 2.0 * dot(dir, v);
	float C = dot(v, v) - sphere.w * sphere.w;
	float B2 = B*B;

	float f = B2 - 4.0 * C;

	if (f < 0.0)
		return 0.0;

	float t0 = -B + sqrt(f);
	float t1 = -B - sqrt(f);
	float t = min(max(t0, 0.0), max(t1, 0.0)) * 0.5;

	if (t == 0.0)
		return 0.0;

	new_pos = from + t*dir;
	new_normal = normalize(new_pos - sphere.xyz);

	return t;
}

vec3 calcLight(vec3 pos, vec3 normal, vec3 color)
{
	vec3 result = vec3(0,0,0);

	for (int i = 0; i < 4; ++i)
	{
		float diff = clamp(dot(normalize(lights[i].position.xyz-pos), normal), 0, 1);
		result += color * diff * lights[i].color.xyz;
	}

	return result;
}
in vec3 dir;
in vec3 pos;

in vec2 fragment_uv;
out vec4 color;

void main (void)
{
	vec4 sky = texture(texture0, dir);
	float diff = 0.0;

	vec3 f_pos = vec3(0,0,0);
	vec3 f_normal = vec3(0, 0, 0);

	vec4 back_color = sky;
	vec4 front_color = vec4(0, 0, 0,0);
	float mint = 10000.0f;

	for (int i = 0; i < 9; ++i)
	{
		float t = intersect_ray_sphere(pos, normalize(dir), spheres[i].position, f_pos, f_normal);
		if (t > 0)
		{
			if (t < mint)
			{
				front_color = vec4(calcLight(f_pos, f_normal, spheres[i].color.rgb), 1.0);

				vec3 mirror = texture(texture0, normalize(reflect(normalize(normalize(dir)), normalize(f_normal)))).rgb;
				float mint2 = 10000.0f;

				vec3 f_pos2 = vec3(0, 0, 0);
				vec3 f_normal2 = vec3(0, 0, 0);
				vec4 front_color2 = vec4(0, 0, 0, 0);

				for (int j = 0; j < 9; ++j)
				{
					float t2 = intersect_ray_sphere(f_pos, f_normal, spheres[j].position, f_pos2, f_normal2);
					if (t2 > 0)
					{
						if (t2 < mint2)
						{
							front_color2 = vec4(calcLight(f_pos2, f_normal2, spheres[j].color.rgb), 1.0);
							vec3 mirror2 = texture(texture0, normalize(reflect(normalize(normalize(dir)), normalize(f_normal2)))).rgb;
							front_color2.rgb += mirror2.rgb * 0.33;
						}
					}
				}

				front_color.rgb += mix(mirror.rgb, front_color2.rgb, front_color2.a) *0.33;
				mint = t;
			}
		}
	}
	

	color = mix(back_color,vec4(front_color.rgb,1.0),front_color.a);
}